# -*- coding: utf-8 -*-
"""
    API - Strawberry Types
"""

import dataclasses as dc
import datetime
import decimal
import typing

# Create your <types> here.
from fastberry import JSON, Model, Text

model = Model()

@model.type
class Author:
    name: str

@model.type
class Product:
    """
    query MyQuery {
        demoDetail {
            name
            aliases
            stock
            isAvailable
            createdAt
            sameDayShippingBefore
            price
            notes
            isObject
            category {
                name
            }
        }
    }
    """
    name: str
    aliases: list[str] | None = None
    stock: int | None = None
    is_available: bool | None = None
    available_from: datetime.date | None = None
    created_at: datetime.datetime | None = None
    same_day_shipping_before: datetime.time | None = None
    price: decimal.Decimal | None = None
    notes: list[Text] = dc.field(default_factory=list)
    is_object: JSON = dc.field(default_factory=dict)

    async def category(self) -> typing.Optional["Category"]:
        return Category(name="awesome")


@model.type
class Category:
    name: str



"""
    Strawberry Pagination
"""

import typing
import dataclasses as dc

import strawberry

from ..config import Settings

settings = Settings()

try:
    ITEMS_PER_PAGE = settings.base.querying.get("items_per_page")
except:
    ITEMS_PER_PAGE = 50


@strawberry.input
@dc.dataclass
class Pagination:
    """GraphQL Pagination"""

    page: int = 1
    limit: typing.Optional[int] = ITEMS_PER_PAGE
    sort_by: typing.Optional[str] = "-id"
    all: typing.Optional[bool] = False

    def __post_init__(self):
        """Add Search Parameters to Query"""
        page = settings.apps.pagination(
            page=self.page,
            limit=self.limit,
        )
        self.page=page.page
        self.limit=page.limit



# -*- coding: utf-8 -*-
"""
    API - CRUD
"""

from fastberry import GQL
from fastberry.graphql.inputs import Pagination
from fastberry.graphql.types import (
    Edges,
    Error,
    ErrorMessage,
    Mutation,
    Query,
    Response,
)

from .. import types


class Demo(GQL):
    """Demo Api"""

    class Query:
        """Query"""

        async def detail() -> Query(types.Product):
            return types.Product(name="Model", aliases=["type", "class", "object"])

        async def single_instance(
            search: str | None = None
        ) -> Query(types.Author):
            return types.Author(_id=1, id=1, name="Ludwig Van Beethoven")

        async def multiple_instances(
            pagination: Pagination,
        ) -> Edges(types.Author):
            print(pagination)
            return Response(
                edges=[
                    types.Author(_id=1, id=1, name="Ludwig Van Beethoven"),
                    types.Author(_id=2, id=2, name="Wolfgang Amadeus Mozart"),
                ],
                length=2,
                pages=1,
            )

    class Mutation:
        """Mutation"""

        async def create_good() -> Mutation(types.Author):
            if True:
                return types.Author(_id=2, id=2, name="Wolfgang Amadeus Mozart")
            return None

        async def create_bad() -> Mutation(types.Author):
            if False:
                return None
            return Error([ErrorMessage(type="input", message="invalid input.")])
